@@ -1,15 +1,21 @@
 package Pod::Weaver::Plugin::Bencher::Scenario;
 
-our $DATE = '2016-01-13'; # DATE
-our $VERSION = '0.05'; # VERSION
+our $DATE = '2016-01-19'; # DATE
+our $VERSION = '0.06'; # VERSION
 
 use 5.010001;
 use Moose;
 with 'Pod::Weaver::Role::AddTextToSection';
 with 'Pod::Weaver::Role::Section';
 
-use Perinci::Result::Format::Lite;
+has sample_bench => (is=>'rw');
+
+sub mvp_multivalue_args { qw(sample_bench) }
+
 use Bencher;
+use List::MoreUtils qw(firstidx);
+use Perinci::Sub::ConvertArgs::Argv qw(convert_args_to_argv);
+use String::ShellQuote;
 
 sub __ver_or_vers {
     my $v = shift;
@@ -102,27 +108,61 @@ sub _process_module {
     my @modules = Bencher::_get_participant_modules($scenario);
 
     # add Sample Benchmark Results section
-    my ($bench_res, $fres, $f2res);
+    my @bench_res;
     {
+        my $fres;
         my @pod;
 
-        $bench_res = Bencher::bencher(
-            action => 'bench',
-            scenario_module => $scenario_name,
-        );
-        $fres = Bencher::format_result($bench_res);
-        $f2res = Perinci::Result::Format::Lite::format($fres, 'text-pretty');
-        $f2res =~ s/^/ /gm;
+        my $sample_benches;
+        if ($self->sample_bench && @{ $self->sample_bench }) {
+            $sample_benches = [];
+            my $i = -1;
+            for (@{ $self->sample_bench }) {
+                $i++;
+                my $res = eval $_;
+                $self->log_fatal(["Invalid sample_bench[$i] specification: %s", $@]) if $@;
 
-        my $num_cores = $bench_res->[3]{'func.cpu_info'}[0]{number_of_cores};
-        push @pod, "Run on: ",
-            "perl: I<< ", __ver_or_vers($bench_res->[3]{'func.module_versions'}{perl}), " >>, ",
-            "CPU: I<< ", $bench_res->[3]{'func.cpu_info'}[0]{name}, " ($num_cores cores) >>, ",
-            "OS: I<< ", $bench_res->[3]{'func.platform_info'}{osname}, " ", $bench_res->[3]{'func.platform_info'}{oslabel}, " version ", $bench_res->[3]{'func.platform_info'}{osvers}, " >>, ",
-            "OS kernel: I<< ", $bench_res->[3]{'func.platform_info'}{kname}, " version ", $bench_res->[3]{'func.platform_info'}{kvers}, " >>",
-            ".\n\n";
+                my $cres = convert_args_to_argv(args => $res->{args}, meta => $Bencher::SPEC{bencher});
+                $self->log_fatal(["Invalid sample_bench[$i] specification: invalid args: %s - %s", $cres->[0], $cres->[1]])
+                    unless $cres->[0] == 200;
+                my $cmd = "C<< bencher ".join(" ", map {shell_quote($_)} @{$cres->[2]})." >>";
+                if ($res->{title}) {
+                    $res->{title} .= " ($cmd)";
+                } else {
+                    $res->{title} = "Benchmark with $cmd";
+                }
+                push @$sample_benches, $res;
+            }
+        } else {
+            $sample_benches = [
+                {title=>"Benchmark with default options", args=>{}},
+            ];
+        }
 
-        push @pod, "Benchmark with default option:\n\n", $f2res, "\n\n";
+        my $i = -1;
+        for my $bench (@$sample_benches) {
+            $i++;
+            my $bench_res = Bencher::bencher(
+                action => 'bench',
+                scenario_module => $scenario_name,
+                %{ $bench->{args} },
+            );
+            $fres = Bencher::format_result($bench_res);
+            $fres =~ s/^/ /gm;
+
+            if ($i == 0) {
+                my $num_cores = $bench_res->[3]{'func.cpu_info'}[0]{number_of_cores};
+                push @pod, "Run on: ",
+                    "perl: I<< ", __ver_or_vers($bench_res->[3]{'func.module_versions'}{perl}), " >>, ",
+                    "CPU: I<< ", $bench_res->[3]{'func.cpu_info'}[0]{name}, " ($num_cores cores) >>, ",
+                    "OS: I<< ", $bench_res->[3]{'func.platform_info'}{osname}, " ", $bench_res->[3]{'func.platform_info'}{oslabel}, " version ", $bench_res->[3]{'func.platform_info'}{osvers}, " >>, ",
+                    "OS kernel: I<< ", $bench_res->[3]{'func.platform_info'}{kname}, " version ", $bench_res->[3]{'func.platform_info'}{kvers}, " >>",
+                    ".\n\n";
+            }
+
+            push @pod, "$bench->{title}:\n\n$fres\n\n";
+            push @bench_res, $bench_res;
+        } # for sample_benches
 
         if (@modules && !$scenario->{module_startup}) {
             my $bench_res2 = Bencher::bencher(
@@ -131,9 +171,8 @@ sub _process_module {
                 scenario_module => $scenario_name,
             );
             $fres = Bencher::format_result($bench_res2);
-            $f2res = Perinci::Result::Format::Lite::format($fres, 'text-pretty');
-            $f2res =~ s/^/ /gm;
-            push @pod, "Benchmark module startup overhead:\n\n", $f2res, "\n\n";
+            $fres =~ s/^/ /gm;
+            push @pod, "Benchmark module startup overhead:\n\n", $fres, "\n\n";
         }
 
         $self->add_text_to_section(
@@ -160,7 +199,7 @@ sub _process_module {
 
         for my $mod (@modules) {
             push @pod, "L<$mod>";
-            my $v = $bench_res->[3]{'func.module_versions'}{$mod};
+            my $v = firstidx {$_->[3]{'func.module_versions'}{$mod}} @bench_res;
             if ($v) {
                 push @pod, " ", __ver_or_vers($v);
             }
@@ -221,6 +260,25 @@ sub _process_module {
             });
     }
 
+    # add Benchmarked Datasets section
+    {
+        last unless $scenario->{datasets} && @{ $scenario->{datasets} };
+        my @pod;
+
+        push @pod, "=over\n\n";
+        for my $ds (@{ $scenario->{datasets} }) {
+            push @pod, "=item * $ds->{name}\n\n";
+            push @pod, "$ds->{summary}\n\n" if $ds->{summary};
+        }
+        push @pod, "=back\n\n";
+
+        $self->add_text_to_section(
+            $document, join("", @pod), 'BENCHMARK DATASETS',
+            {
+                after_section => 'BENCHMARK PARTICIPANTS',
+            });
+    }
+
     $self->log(["Generated POD for '%s'", $filename]);
 }
 
@@ -240,8 +298,6 @@ sub weave_section {
 1;
 # ABSTRACT: Plugin to use when building Bencher::Scenario::* distribution
 
-
-
 __END__
 
 =pod
@@ -254,7 +310,7 @@ Pod::Weaver::Plugin::Bencher::Scenario - Plugin to use when building Bencher::Sc
 
 =head1 VERSION
 
-This document describes version 0.05 of Pod::Weaver::Plugin::Bencher::Scenario (from Perl distribution Pod-Weaver-Plugin-Bencher-Scenario), released on 2016-01-13.
+This document describes version 0.06 of Pod::Weaver::Plugin::Bencher::Scenario (from Perl distribution Pod-Weaver-Plugin-Bencher-Scenario), released on 2016-01-19.
 
 =head1 SYNOPSIS
 
@@ -284,11 +340,7 @@ run and shown.
 
 =back
 
-=for Pod::Coverage ^(weave_section)$
-
-=head1 SEE ALSO
-
-L<Dist::Zilla::Plugin::Bencher::Scenario>
+=for Pod::Coverage .*
 
 =head1 HOMEPAGE
 
@@ -306,6 +358,10 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
+L<Dist::Zilla::Plugin::Bencher::Scenario>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
@@ -318,4 +374,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
